import os
from typing import Any, cast

from mongoengine import Document, connect
from mongoengine.fields import IntField, StringField


class TranscodeItem(Document):
    objects: Any

    path = StringField(required=True)

    languages = StringField(required=False)

    video_params = StringField(required=False)
    audio_params = StringField(required=False)

    content_id = IntField(required=False)


class TranscodeQueue:
    def __init__(self) -> None:
        connect(
            "wi1_bot",
            connect=True,
            host=os.environ.get("MONGODB_CONNECTION_STRING", None),
            uuidrepresentation="standard",
        )

    def add(
        self,
        path: str,
        languages: str | None = None,
        video_params: str | None = None,
        audio_params: str | None = None,
        content_id: int | None = None,
    ) -> None:
        TranscodeItem(
            path=path,
            languages=languages,
            video_params=video_params,
            audio_params=audio_params,
            content_id=content_id,
        ).save()

    def get_one(self) -> TranscodeItem | None:
        return cast(TranscodeItem | None, TranscodeItem.objects.first())

    def remove(self, item: TranscodeItem) -> None:
        item.delete()

    def clear(self) -> None:
        TranscodeItem.objects.delete()

    @property
    def size(self) -> int:
        return cast(int, TranscodeItem.objects.count())


queue = TranscodeQueue()



import os
import pathlib
from typing import NotRequired, TypedDict

import yaml

_config_path = os.getenv("WB_CONFIG_PATH")

if _config_path is None:
    if home := os.getenv("HOME"):
        _path = pathlib.Path(home) / ".config" / "wi1-bot" / "config.yaml"
        if _path.is_file():
            _config_path = str(_path.resolve())

    if xdg_config_home := os.getenv("XDG_CONFIG_HOME"):
        _path = pathlib.Path(xdg_config_home) / "wi1-bot" / "config.yaml"
        if _path.is_file():
            _config_path = str(_path.resolve())

    if pathlib.Path("config.yaml").is_file():
        _config_path = "config.yaml"

if _config_path is None:
    raise FileNotFoundError(
        "could not find ./config.yaml, $XDG_CONFIG_HOME/wi1-bot/config.yaml or"
        " $HOME/.config/wi1-bot/config.yaml"
    )


class ArrConfig(TypedDict):
    url: str
    api_key: str
    root_folder: str


class PushoverConfig(TypedDict):
    user_key: str
    api_key: str
    devices: str


class DiscordConfig(TypedDict):
    bot_token: str
    channel_id: int
    admin_id: int
    bot_presence: NotRequired[str]
    quotas: NotRequired[dict[int, float]]


class TranscodingProfile(TypedDict):
    video_params: NotRequired[str]
    audio_params: NotRequired[str]
    languages: NotRequired[str]


class TranscodingConfig(TypedDict):
    profiles: dict[str, TranscodingProfile]
    hwaccel: NotRequired[str]


class RemotePathMapping(TypedDict):
    remote: str
    local: str


class GeneralConfig(TypedDict):
    log_dir: NotRequired[str]
    remote_path_mappings: NotRequired[list[RemotePathMapping]]


class Config(TypedDict):
    general: NotRequired[GeneralConfig]  # optional
    radarr: ArrConfig
    sonarr: ArrConfig
    discord: DiscordConfig
    pushover: NotRequired[PushoverConfig]
    transcoding: NotRequired[TranscodingConfig]


with open(_config_path, "r") as f:
    config: Config = yaml.load(f, Loader=yaml.SafeLoader)

if log_dir := os.getenv("WB_LOG_DIR"):
    if "general" not in config:
        config["general"] = {}

    config["general"]["log_dir"] = log_dir



import logging
import pathlib

from wi1_bot.config import RemotePathMapping, config


def replace_remote_paths(path: pathlib.Path) -> pathlib.Path:
    if "general" not in config or "remote_path_mappings" not in config["general"]:
        return path

    mappings = config["general"]["remote_path_mappings"]

    most_specific: RemotePathMapping | None = None

    for mapping in mappings:
        if path.is_relative_to(mapping["remote"]):
            mapping_len = len(pathlib.Path(mapping["remote"]).parts)
            most_specific_len = (
                len(pathlib.Path(most_specific["remote"]).parts) if most_specific is not None else 0
            )

            if mapping_len > most_specific_len:
                most_specific = mapping

    if most_specific is not None:
        remote_path = path
        path = pathlib.Path(most_specific["local"]) / path.relative_to(most_specific["remote"])

        logging.getLogger(__name__).debug(f"replaced remote path mapping: {remote_path} -> {path}")

    return path



import pathlib
import shlex
import shutil

import pytest

from wi1_bot.transcoder import Transcoder, ffprobe
from wi1_bot.transcoder.transcode_queue import TranscodeItem
from wi1_bot.transcoder.transcoder import build_ffmpeg_command

FILES_PATH = pathlib.Path("./tests/files")


@pytest.fixture(autouse=True)
def setup_files():
    for file in FILES_PATH.iterdir():
        if file.is_file() and file.suffix in [".mkv", ".mp4"]:
            shutil.copy(file, file.with_name(f"{file.name}.bak"))

    yield

    for file in FILES_PATH.glob("*-TRANSCODED.mkv"):
        file.unlink(missing_ok=True)

    for file in FILES_PATH.glob("*.bak"):
        file.rename(file.with_name(file.stem))


def test_copy_mjpeg():
    path = FILES_PATH / "h264_eac3_pgssub_mjpeg.mkv"

    item = TranscodeItem(
        path=str(path),
        video_params="-c libx264 -b 2000k",
        audio_params="-c aac -b 128k -ac 2",
    )

    t = Transcoder()
    print(shlex.join(build_ffmpeg_command(item, "output.mkv")))
    t.transcode(item)

    transcoded = path.with_name(f"{path.stem}-TRANSCODED.mkv")
    assert transcoded.exists()

    output = ffprobe(transcoded)
    streams = output["streams"]
    assert isinstance(streams, list)
    assert any(s["codec_type"] == "video" and s["codec_name"] == "mjpeg" for s in streams)


def test_convert_movtext():
    path = FILES_PATH / "h264_eac3_movtext.mp4"

    item = TranscodeItem(path=str(path))

    t = Transcoder()
    print(shlex.join(build_ffmpeg_command(item, "output.mkv")))
    t.transcode(item)

    transcoded = path.with_name(f"{path.stem}-TRANSCODED.mkv")
    assert transcoded.exists()

    output = ffprobe(transcoded)
    streams = output["streams"]
    assert isinstance(streams, list)
    assert any(s["codec_type"] == "subtitle" and s["codec_name"] == "subrip" for s in streams)
